--
-- Copyright (C) 1988-1999 Altera Corporation
--
-- Any megafunction design, and related net list (encrypted or decrypted),
-- support information, device programming or simulation file, and any
-- other associated documentation or information provided by Altera or a
-- partner under Altera's Megafunction Partnership Program may be used only
-- to program PLD devices (but not masked PLD devices) from Altera.  Any
-- other use of such megafunction design, net list, support information,
-- device programming or simulation file, or any other related
-- documentation or information is prohibited for any other purpose,
-- including, but not limited to modification, reverse engineering, de-
-- compiling, or use with any other silicon devices, unless such use is
-- explicitly licensed under a separate agreement with Altera or a
-- megafunction partner.  Title to the intellectual property, including
-- patents, copyrights, trademarks, trade secrets, or maskworks, embodied
-- in any such megafunction design, net list, support information, device
-- programming or simulation file, or any other related documentation or
-- information provided by Altera or a megafunction partner, remains with
-- Altera, the megafunction partner, or their respective licensors.  No
-- other licenses, including any licenses needed under any third party's
-- intellectual property, are provided herein.
----------------------------------------------------------------------------
----------------------------------------------------------------------------
-- ALtera Megafunction Component Declaration File
--
-- Version QuartusII 1.0 Date: 01/09/01
--
-- 01/09/01: Fix SPR 79392 for altcdr_rx.
-- 01/05/01: Default value of altlvds_tx outclock_divide_by set to zero.
-- 01/04/01: Fix sprs 79179 & 79181 for altddio and altcdr.
--           Also added missing lcell, global, carry and cascade
-- 11/27/00: Fix SPR75474. Add tx_pll_enable and rx_pll_enable to altlvds_tx
--           and altlvds_rx respectively.
-- 11/27/00: Fix SPR75144. Mercury altlvds_tx and new port tx_coreclock.
-- New components: 
-- 	alt_exc_dpram, alt_exc_upcore, altdido_in, altdido_out,
--	altdido_bidir, altcdr_rx, altcdr_tx 
----------------------------------------------------------------------------
-- Version 1.01 Date: 11/08/00
--
-- 11/08/00: Added alt_exc_dpram and alt_exc_upcore.
-- 11/03/00: Changed default values of altqpram parameters OUTDATA_ACLR_*,
--           RDCONTROL_ACLR_*, RDADDRESS_ACLR_*.  Also removed parameter
--           USE_EAB since it is always on for altqpram.
-- 19/10/00: And provide workaround for when mstart and mnext
--           are unused in fast multiple mode. See SPR76920. 
-- 08/21/00: Added default values to aclr and sclr of dcfifo and scfifo.
-- 08/18/00: Added INTENDED_DEVICE_FAMILY parameter to altdpram.
-- 08/14/00: Valid values for *_ACLR_* parameters of altqpram have changed
--           from ON and OFF to INACLR_A (or INACLR_B) and NONE
--           respectively.
-- 08/14/00: SPR 74650
--           Removed several internal-use only parameters (MAXIMUM_DEPTH,
--           CASCADE_CHAIN, DEVICE_FAMILY, DATA_INTERLEAVE_OFFSET_IN_BITS,
--           and DATA_INTERLEAVE_WIDTH_IN_BITS) from altqpram so that
--           Quartus won't behave strangely.  Also removed DEVICE_FAMILY
--           and OPTIMIZE_FOR_SPEED parameters from scfifo.
-- 08/03/00: Added default values to all altqpram ports and parameters.
-- 08/02/00: Removed MAXIMIZE_SPEED, ALMOST_FULL_VALUE, ALMOST_EMPTY_VALUE,
--           and ALLOW_RWCYCLE_WHEN_FULL parameters from dcfifo.  Added
--           LPM_HINT parameter to dcfifo.
-- 08/02/00: Removed MAXIMIZE_SPEED parameter from scfifo.
-- 07/29/00: Added WRADDRESS_ACLR_A, WRADDRESS_ACLR_B, and LPM_HINT
--           parameters to altqpram.
-- 07/29/00: Added default value to CASCADE_CHAIN and DEVICE_FAMILY
--           parameters of altqpram.
-- 07/28/00: INCLOCK_PERIOD defaulted to "10000". See SPR73860.
-- 07/26/00: Update altdpram to use "lpm_file" and "lpm_hint".
--           Latest altlvds_rx and altlvds_tx dfactor changes added.
-- 07/17/00: Changing altclklock *time_delay* parameter to string type to
--           allow for negative values.
-- 06/26/00: adding "operation_mode" param to altlvds_rx&tx.
-- 06/23/00: Adding parameter intended_device_family to altclklock, altlvds_rx
--           and altlvds_tx
-- 06/21/00: Changed all parameters of scfifo and dcfifo from type positive
--             to type natural.
-- 06/15/00: Significant fixes to altclklock, altlvds_rx and altlvds_rx.
--           SPR 71839, 72411 and 72570. Corrected bit-ordering, deskew
--           usage, issues with "Duty Cycle violation" etc.
-- 06/02/00: Added "jumbo" fix for altcam. SPR 70735, 71239, 71240, 71614
--           71617, 71940. Passes all regtest. NB: The state of the outputs
--           during a MULTIPLE mode write cycle is not modelled. (trying to
--           determine what correct behavior is; this behavior is of no 
--           importance to users.)
-- 04/20/00: Added component declaration of altqpram.
-- 04/19/00: SPR 70899
--           Added component declarations for "scfifo" and "dcfifo".
----------------------------------------------------------------------------
-- Version 1.00 Date: 04/05/00
--
-- 04/05/00: SPR 69196
--           Rename of "apex20ke_mf_components" to "altera_mf_components".
--           Added component declaration of "altdpram".
-- 03/08/00: SPR 68829
--           Added use_eab parameter to altcam
----------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;

package altera_mf_components is
component altcam
   generic
      ( width:            natural := 1;
        widthad:          natural := 1;
        numwords:         natural := 1;
        lpm_file:         string := "UNUSED";
        lpm_filex:        string := "UNUSED";
        match_mode:       string := "MULTIPLE";
        output_reg:       string := "UNREGISTERED";
        output_aclr:      string := "OFF";
        pattern_reg:      string := "INCLOCK";
        pattern_aclr:     string := "ON";
        wraddress_aclr:   string := "ON";
        wrx_reg:          string := "UNUSED";
        wrx_aclr:         string := "UNUSED";
        wrcontrol_aclr:   string := "OFF";
        use_eab:          string := "ON" );
        
   port
      ( pattern:    in std_logic_vector(width-1 downto 0);
        wrx:        in std_logic_vector(width-1 downto 0) := (others => '0');
        wrxused:    in std_logic := '1';
        wrdelete:   in std_logic := '0';
        wraddress:  in std_logic_vector(widthad-1 downto 0);
        wren:       in std_logic;
        inclock:    in std_logic;
        inclocken:  in std_logic := '1';
        inaclr:     in std_logic := '0';
        mstart:     in std_logic := 'X';
        mnext:      in std_logic := '0';
        outclock:   in std_logic := '0';
        outclocken: in std_logic := '1';
        outaclr:    in std_logic := '0';
        maddress:   out std_logic_vector(widthad-1 downto 0);
        mbits:      out std_logic_vector(numwords-1 downto 0);
        mfound:     out std_logic;
        mcount:     out std_logic_vector(widthad-1 downto 0);
        rdbusy:     out std_logic;
        wrbusy:     out std_logic );
end component;


component altclklock
    generic
      ( inclock_period          : natural := 10000;
        inclock_settings        : string := "UNUSED";
        valid_lock_cycles       : natural := 5;
        invalid_lock_cycles     : natural := 5;
        valid_lock_multiplier   : natural := 1;
        invalid_lock_multiplier : natural := 1;
        operation_mode          : string := "NORMAL";
        clock0_boost            : natural := 1;
        clock0_divide           : natural := 1;
        clock1_boost            : natural := 1;
        clock1_divide           : natural := 1;
        clock2_boost            : natural := 1;
        clock2_divide           : natural := 1;
        clock_ext_boost         : natural := 1;
        clock_ext_divide        : natural := 1;
        clock0_settings         : string := "UNUSED";
        clock1_settings         : string := "UNUSED";
        clock2_settings         : string := "UNUSED";
        clock_ext_settings      : string := "UNUSED";
        outclock_phase_shift    : natural := 0;
        clock0_time_delay       : string := "0";
        clock1_time_delay       : string := "0";
        clock2_time_delay       : string := "0";
        clock_ext_time_delay    : string := "0";
        intended_device_family  : string := "APEX20KE");
    port
      ( inclock   : in std_logic;
        inclocken : in std_logic := '1';
        fbin      : in std_logic := '1';
        clock0    : out std_logic;
        clock1    : out std_logic;
        clock2    : out std_logic;
        clock_ext : out std_logic;
        locked    : out std_logic );
end component;


component altlvds_rx
    generic
      ( number_of_channels     : natural;
        deserialization_factor : natural;
        registered_output      : string := "ON";
        inclock_period         : natural;
        clock_setting          : string := "UNUSED";
        intended_device_family : string := "APEX20KE");
    port
      ( rx_in           : in std_logic_vector(number_of_channels-1 downto 0);
        rx_inclock      : in std_logic;
        rx_deskew       : in std_logic := '0';
        rx_pll_enable   : in std_logic := '1';
        rx_out          : out std_logic_vector(deserialization_factor*number_of_channels -1 downto 0);
        rx_outclock     : out std_logic;
        rx_locked       : out std_logic );
end component;

component altlvds_tx
    generic
      ( number_of_channels     : natural;
        deserialization_factor : natural;
        outclock_divide_by     : natural := 0;
        registered_input       : string := "ON";
        multi_clock            : string := "OFF";
        inclock_period         : natural;
        clock_setting          : string := "UNUSED";
        intended_device_family : string := "APEX20KE");
    port
      ( tx_in           : in std_logic_vector(deserialization_factor*number_of_channels -1 downto 0);
        tx_inclock      : in std_logic;
        sync_inclock    : in std_logic := '0';
        tx_pll_enable   : in std_logic := '1';
        tx_out          : out std_logic_vector(number_of_channels-1 downto 0);
        tx_outclock     : out std_logic;
        tx_coreclock    : out std_logic;
        tx_locked       : out std_logic );
end component;

component altdpram
    generic
      ( width                  : natural;
        widthad                : natural;
        numwords               : natural := 0;
        lpm_file               : string := "UNUSED";
        lpm_hint               : string := "USE_EAB=ON";
        indata_reg             : string := "UNREGISTERED";
        indata_aclr            : string := "OFF";
        wraddress_reg          : string := "UNREGISTERED";
        wraddress_aclr         : string := "OFF";
        wrcontrol_reg          : string := "UNREGISTERED";
        wrcontrol_aclr         : string := "OFF";
        rdaddress_reg          : string := "UNREGISTERED";
        rdaddress_aclr         : string := "OFF";
        rdcontrol_reg          : string := "UNREGISTERED";
        rdcontrol_aclr         : string := "OFF";
        outdata_reg            : string := "UNREGISTERED";
        outdata_aclr           : string := "OFF";
        intended_device_family : string := "APEX20KE");
    port
      ( wren       : in std_logic := '0';
        data       : in std_logic_vector(width-1 downto 0);
        wraddress  : in std_logic_vector(widthad-1 downto 0);
        inclock    : in std_logic := '0';
        inclocken  : in std_logic := '1';
        rden       : in std_logic := '1';
        rdaddress  : in std_logic_vector(widthad-1 downto 0);
        outclock   : in std_logic := '0';
        outclocken : in std_logic := '1';
        aclr       : in std_logic := '0';
        q          : out std_logic_vector(width-1 downto 0) );
end component;


component altqpram
    generic
      ( operation_mode            : string := "QUAD_PORT";

        width_write_a             : natural := 1;
        widthad_write_a           : natural := 1;
        numwords_write_a          : natural := 2;  -- default = 2^widthad_write_a
        indata_reg_a              : string := "INCLOCK_A";
        indata_aclr_a             : string := "INACLR_A";
        wrcontrol_wraddress_reg_a : string := "INCLOCK_A";
        wrcontrol_aclr_a          : string := "INACLR_A";
        wraddress_aclr_a          : string := "INACLR_A";

        width_write_b             : natural := 1;  -- default = width_write_a
        widthad_write_b           : natural := 1;  -- default = widthad_write_a
        numwords_write_b          : natural := 2;  -- default = 2^widthad_write_b
        indata_reg_b              : string := "INCLOCK_B";
        indata_aclr_b             : string := "INACLR_B";
        wrcontrol_wraddress_reg_b : string := "INCLOCK_B";
        wrcontrol_aclr_b          : string := "INACLR_B";
        wraddress_aclr_b          : string := "INACLR_B";

        width_read_a              : natural := 1;
        widthad_read_a            : natural := 1;
        numwords_read_a           : natural := 2;  -- default = 2^widthad_read_a
        rdcontrol_reg_a           : string := "OUTCLOCK_A";
        rdcontrol_aclr_a          : string := "OUTACLR_A";
        rdaddress_reg_a           : string := "OUTCLOCK_A";
        rdaddress_aclr_a          : string := "OUTACLR_A";
        outdata_reg_a             : string := "UNREGISTERED";
        outdata_aclr_a            : string := "OUTACLR_A";

        width_read_b              : natural := 1;  -- default = width_read_a
        widthad_read_b            : natural := 1;  -- default = widthad_read_a
        numwords_read_b           : natural := 2;  -- default = 2^widthad_read_b
        rdcontrol_reg_b           : string := "OUTCLOCK_B";
        rdcontrol_aclr_b          : string := "OUTACLR_B";
        rdaddress_reg_b           : string := "OUTCLOCK_B";
        rdaddress_aclr_b          : string := "OUTACLR_B";
        outdata_reg_b             : string := "UNREGISTERED";
        outdata_aclr_b            : string := "OUTACLR_B";

        init_file                 : string := "UNUSED";
        lpm_hint                  : string := "UNUSED" );

    port
      ( wren_a       : in std_logic := '0';
        wren_b       : in std_logic := '0';
        data_a       : in std_logic_vector(width_write_a-1 downto 0) := (OTHERS => '0');
        data_b       : in std_logic_vector(width_write_b-1 downto 0) := (OTHERS => '0');
        wraddress_a  : in std_logic_vector(widthad_write_a-1 downto 0) := (OTHERS => '0');
        wraddress_b  : in std_logic_vector(widthad_write_b-1 downto 0) := (OTHERS => '0');
        inclock_a    : in std_logic := '1';
        inclock_b    : in std_logic := '1';
        inclocken_a  : in std_logic := '1';
        inclocken_b  : in std_logic := '1';
        rden_a       : in std_logic := '1';
        rden_b       : in std_logic := '1';
        rdaddress_a  : in std_logic_vector(widthad_read_a-1 downto 0) := (OTHERS => '0');
        rdaddress_b  : in std_logic_vector(widthad_read_b-1 downto 0) := (OTHERS => '0');
        outclock_a   : in std_logic := '1';
        outclock_b   : in std_logic := '1';
        outclocken_a : in std_logic := '1';
        outclocken_b : in std_logic := '1';
        inaclr_a     : in std_logic := '0';
        inaclr_b     : in std_logic := '0';
        outaclr_a    : in std_logic := '0';
        outaclr_b    : in std_logic := '0';
        q_a          : out std_logic_vector(width_read_a-1 downto 0);
        q_b          : out std_logic_vector(width_read_b-1 downto 0) );
end component;

component scfifo
    generic
      ( lpm_width               : natural;
        lpm_widthu              : natural;
        lpm_numwords            : natural;
        lpm_showahead           : string := "OFF";
        lpm_hint                : string := "USE_EAB=ON";
        almost_full_value       : natural := 0;
        almost_empty_value      : natural := 0;
        overflow_checking       : string := "ON";
        underflow_checking      : string := "ON";
        allow_rwcycle_when_full : string := "OFF";
        -- parameters that should be stored in lpm_hint
        use_eab                 : string := "ON");
    port
      ( data         : in std_logic_vector(lpm_width-1 downto 0);
        clock        : in std_logic;
        wrreq        : in std_logic;
        rdreq        : in std_logic;
        aclr         : in std_logic := '0';
        sclr         : in std_logic := '0';
        full         : out std_logic;
        almost_full  : out std_logic;
        empty        : out std_logic;
        almost_empty : out std_logic;
        q            : out std_logic_vector(lpm_width-1 downto 0);
        usedw        : out std_logic_vector(lpm_widthu-1 downto 0)
     );
end component;

component dcfifo
    generic
      ( lpm_width               : natural;
        lpm_widthu              : natural;
        lpm_numwords            : natural;
        lpm_showahead           : string := "OFF";
        lpm_hint                : string := "USE_EAB=ON";
        overflow_checking       : string := "ON";
        underflow_checking      : string := "ON";
        delay_rdusedw           : natural := 1;
        delay_wrusedw           : natural := 1;
        rdsync_delaypipe        : natural := 3;
        wrsync_delaypipe        : natural := 3;
        -- parameters that should be stored in lpm_hint
        use_eab                 : string := "ON";
        clocks_are_synchronized : string := "FALSE");
    port
      ( data    : in std_logic_vector(lpm_width-1 downto 0);
        rdclk   : in std_logic;
        wrclk   : in std_logic;
        wrreq   : in std_logic;
        rdreq   : in std_logic;
        aclr    : in std_logic := '0';
        rdfull  : out std_logic;
        wrfull  : out std_logic;
        wrempty : out std_logic;
        rdempty : out std_logic;
        q       : out std_logic_vector(lpm_width-1 downto 0);
        rdusedw : out std_logic_vector(lpm_widthu-1 downto 0);
        wrusedw : out std_logic_vector(lpm_widthu-1 downto 0));
end component;

component alt_exc_dpram
    generic 
      ( width          : integer;
        widthad        : integer;
        depth          : integer;
        ramblock       : integer := 65535;
        operation_mode : string := "SINGLE_PORT";
        output_mode    : string := "REG";
        lpm_file       : string := "NONE");

    port
      ( portaclk : in std_logic := '0';
        portaena : in std_logic := '1';
        portawe : in std_logic := '0';
        portaaddr : in std_logic_vector(widthad-1 downto 0) := (others =>'0');
        portadatain : in std_logic_vector(width-1 downto 0) := (others =>'0');
        portadataout : out std_logic_vector(width-1 downto 0);
        portbclk : in std_logic := '0';
        portbena : in std_logic := '1';
        portbwe : in std_logic := '0';
        portbaddr : in std_logic_vector(widthad-1 downto 0) := (others =>'0');
        portbdatain : in std_logic_vector(width-1 downto 0) := (others =>'0');
        portbdataout : out std_logic_vector(width-1 downto 0));
end component;

component alt_exc_upcore
    generic
      ( processor      : string := "ARM";
        source         : string := "");

    port
      ( intpld    : in  std_logic_vector(5 downto 0) := (others => '0');
        intuart   : out std_logic;
        inttimer0 : out std_logic;
        inttimer1 : out std_logic;
        intproc0  : out std_logic;
        intproc1  : out std_logic;

        debugrq      : in  std_logic := '0';
        debugext0    : in  std_logic := '0';
        debugext1    : in  std_logic := '0';
        debugiebrkpt : in  std_logic := '0';
        debugdewpt   : in  std_logic := '0';
        debugextin   : in  std_logic_vector(3 downto 0) := (others => '0');
        debugack     : out std_logic;
        debugrng0    : out std_logic;
        debugrng1    : out std_logic;
        debugextout  : out std_logic_vector(3 downto 0);

        slavehclk      : in  std_logic := '0';
        slavehwrite    : in  std_logic := '0';
        slavehreadyi   : in  std_logic := '0';
        slavehselreg   : in  std_logic := '0';
        slavehsel      : in  std_logic := '0';
        slavehmastlock : in  std_logic := '0';
        slavehaddr     : in  std_logic_vector(31 downto 0) := (others => '0');
        slavehwdata    : in  std_logic_vector(31 downto 0) := (others => '0');
        slavehtrans    : in  std_logic_vector(1 downto 0) := (others => '0');
        slavehsize     : in  std_logic_vector(1 downto 0) := (others => '0');
        slavehburst    : in  std_logic_vector(2 downto 0) := (others => '0');
        slavehreadyo   : out std_logic;
        slavebuserrint : out std_logic;
        slavehrdata    : out std_logic_vector(31 downto 0);
        slavehresp     : out std_logic_vector(1 downto 0);

        masterhclk     : in  std_logic := '0';
        masterhrdata   : in  std_logic_vector(31 downto 0) := (others => '0');
        masterhresp    : in  std_logic_vector(1 downto 0) := (others => '0');
        masterhwrite   : out std_logic;
        masterhlock    : out std_logic;
        masterhbusreq  : out std_logic;
        masterhaddr    : out std_logic_vector(31 downto 0);
        masterhwdata   : out std_logic_vector(31 downto 0);
        masterhtrans   : out std_logic_vector(1 downto 0);
        masterhsize    : out std_logic_vector(1 downto 0);
        masterhready   : in  std_logic := '0';
        masterhburst   : out std_logic_vector(2 downto 0);
        masterhgrant   : in  std_logic := '1';

        lockreqdp0   : in  std_logic := '0';
        lockreqdp1   : in  std_logic := '0';
        lockgrantdp0 : out std_logic;
        lockgrantdp1 : out std_logic;

        ebiack  : in  std_logic := '0';
        ebiwen  : out std_logic;
        ebioen  : out std_logic;
        ebiclk  : out std_logic;
        ebibe   : out std_logic_vector(1 downto 0);
        ebicsn  : out std_logic_vector(3 downto 0);
        ebiaddr : out std_logic_vector(24 downto 0);

        ebidq : inout std_logic_vector(15 downto 0) := (others => '0');

        uarttxd  : out   std_logic;
        uartrtsn : out   std_logic;
        uartdtrn : out   std_logic;
        uartctsn : in    std_logic := '0';
        uartdsrn : in    std_logic := '0';
        uartrxd  : in    std_logic := '0';
        uartdcdn : inout std_logic := '0';
        uartrin  : inout std_logic := '0';

        sdramclk  : out std_logic;
        sdramclkn : out std_logic;
        sdramclke : out std_logic;
        sdramwen  : out std_logic;
        sdramcasn : out std_logic;
        sdramrasn : out std_logic;
        sdramdqm  : out std_logic_vector(3 downto 0);
        sdramaddr : out std_logic_vector(14 downto 0);
        sdramcsn  : out std_logic_vector(1 downto 0);

        sdramdq  : inout std_logic_vector(31 downto 0) := (others => '0');
        sdramdqs : inout std_logic_vector(3 downto 0) := (others => '0');

        intextpin     : in  std_logic := '0';
        traceclk      : out std_logic;
        tracesync     : out std_logic;
        tracepipestat : out std_logic_vector(2 downto 0);
        tracepkt      : out std_logic_vector(15 downto 0));
end component;

component alt_exc_stripe
  generic
    ( processor        :       string                                   := "ARM";
      device_size      :       integer                                  := 1000;
      boot_from_flash  :       string                                   := "TRUE";
      debug_extensions :       string                                   := "TRUE";
      use_short_reset  :       string                                   := "TRUE";
      use_initialisation_files  :       string                          := "FALSE";
      dp0_mode         :       string                                   := "COMBINEDx32";
      dp1_mode         :       string                                   := "UNUSED";
      dp0_width        :       integer                                  := 32;
      dp0_widthad      :       integer                                  := 15;
      dp1_width        :       integer                                  := 32;
      dp1_widthad      :       integer                                  := 15;
      ebi0_width       :       integer                                  := 16;
      dp0_output_mode  :       string                                   := "UNREG";
      dp1_output_mode  :       string                                   := "UNREG";
      dp0_lpm_file     :       string                                   := "";
      dp1_lpm_file     :       string                                   := "";
      dp2_lpm_file     :       string                                   := "";
      dp3_lpm_file     :       string                                   := "");
  port
    ( clk_ref          : in    std_logic                                := '0';
      nreset           : inout std_logic                                := '1';
      npor             : in    std_logic                                := '1';
      proc_ntrst       : in    std_logic                                := '0';
      proc_tck         : in    std_logic                                := '0';
      proc_tdi         : in    std_logic                                := '0';
      proc_tdo         : out   std_logic;
      proc_tms         : in    std_logic                                := '0';
      ebiack           : in    std_logic                                := '0';
      debugrq          : in    std_logic                                := '0';
      debugext0        : in    std_logic                                := '0';
      debugext1        : in    std_logic                                := '0';
      debugiebrkpt     : in    std_logic                                := '0';
      debugdewpt       : in    std_logic                                := '0';
      lockreqdp0       : in    std_logic                                := '0';
      lockreqdp1       : in    std_logic                                := '0';
      uartctsn         : in    std_logic                                := '0';
      uartdsrn         : in    std_logic                                := '0';
      intextpin        : in    std_logic                                := '0';
      uartrxd          : in    std_logic                                := '0';
      slavehmastlock   : in    std_logic                                := '0';
      slavehsel        : in    std_logic                                := '0';
      slavehselreg     : in    std_logic                                := '0';
      slavehreadyi     : in    std_logic                                := '0';
      slavehwrite      : in    std_logic                                := '0';
      slavehclk        : in    std_logic                                := '0';
      masterhgrant     : in    std_logic                                := '0';
      masterhready     : in    std_logic                                := '0';
      masterhclk       : in    std_logic                                := '0';
      ebiclk           : out   std_logic                                := '0';
      sdramclke        : out   std_logic                                := '0';
      sdramclkn        : out   std_logic                                := '0';
      sdramwen         : out   std_logic                                := '0';
      sdramcasn        : out   std_logic                                := '0';
      sdramrasn        : out   std_logic                                := '0';
      ebiwen           : out   std_logic                                := '0';
      traceclk         : out   std_logic                                := '0';
      tracesync        : out   std_logic                                := '0';
      ebioen           : out   std_logic                                := '0';
      uarttxd          : out   std_logic                                := '0';
      uartrtsn         : out   std_logic                                := '0';
      uartrin          : inout std_logic                                := '0';
      uartdcdn         : inout std_logic                                := '0';
      uartdtrn         : out   std_logic                                := '0';
      sdramclk         : out   std_logic                                := '0';
      debugextin       : in    std_logic_vector(3 downto 0)             := (others => '0');
      ebiaddr          : out   std_logic_vector(24 downto 0);
      ebibe            : out   std_logic_vector(1 downto 0);
      ebicsn           : out   std_logic_vector(3 downto 0);
      ebidq            : inout std_logic_vector(15 downto 0)            := (others => '0');
      intpld           : in    std_logic_vector(5 downto 0)             := (others => '0');
      masterhrdata     : in    std_logic_vector(31 downto 0)            := (others => '0');
      masterhresp      : in    std_logic_vector(1 downto 0)             := (others => '0');
      sdramaddr        : out   std_logic_vector(14 downto 0);
      sdramcsn         : out   std_logic_vector(1 downto 0);
      sdramdq          : inout std_logic_vector(31 downto 0)            := (others => '0');
      sdramdqm         : out   std_logic_vector(3 downto 0);
      sdramdqs         : inout std_logic_vector(3 downto 0)             := (others => '0');
      slavehaddr       : in    std_logic_vector(31 downto 0)            := (others => '0');
      slavehburst      : in    std_logic_vector(2 downto 0)             := (others => '0');
      slavehsize       : in    std_logic_vector(1 downto 0)             := (others => '0');
      slavehtrans      : in    std_logic_vector(1 downto 0)             := (others => '0');
      slavehwdata      : in    std_logic_vector(31 downto 0)            := (others => '0');
      tracepipestat    : out   std_logic_vector(2 downto 0);
      tracepkt         : out   std_logic_vector(15 downto 0);
      slavehreadyo     : out   std_logic;
      lockgrantdp0     : out   std_logic;
      lockgrantdp1     : out   std_logic;
      inttimer0        : out   std_logic;
      inttimer1        : out   std_logic;
      intproc0         : out   std_logic;
      intproc1         : out   std_logic;
      debugack         : out   std_logic;
      debugrng0        : out   std_logic;
      debugrng1        : out   std_logic;
      intuart          : out   std_logic;
      slavebuserrint   : out   std_logic;
      masterhwrite     : out   std_logic;
      masterhlock      : out   std_logic;
      masterhbusreq    : out   std_logic;
      debugextout      : out   std_logic_vector(3 downto 0);
      masterhaddr      : out   std_logic_vector(31 downto 0);
      masterhburst     : out   std_logic_vector(2 downto 0);
      masterhsize      : out   std_logic_vector(1 downto 0);
      masterhtrans     : out   std_logic_vector(1 downto 0);
      masterhwdata     : out   std_logic_vector(31 downto 0);
      slavehrdata      : out   std_logic_vector(31 downto 0);
      slavehresp       : out   std_logic_vector(1 downto 0);
      dp0_2_portaclk   : in    std_logic                                := '0';
      dp0_portaena     : in    std_logic                                := '0';
      dp0_portawe      : in    std_logic                                := '0';
      dp0_portaaddr    : in    std_logic_vector(dp0_widthad-1 downto 0) := (others => '0');
      dp0_portadatain  : in    std_logic_vector(dp0_width-1 downto 0)   := (others => '0');
      dp0_portadataout : out   std_logic_vector(dp0_width-1 downto 0);
      dp0_portbclk     : in    std_logic                                := '0';
      dp0_portbena     : in    std_logic                                := '0';
      dp0_portbwe      : in    std_logic                                := '0';
      dp0_portbaddr    : in    std_logic_vector(dp0_widthad-1 downto 0) := (others => '0');
      dp0_portbdatain  : in    std_logic_vector(dp0_width-1 downto 0)   := (others => '0');
      dp0_portbdataout : out   std_logic_vector(dp0_width-1 downto 0);
      dp1_3_portaclk   : in    std_logic                                := '0';
      dp1_portaena     : in    std_logic                                := '0';
      dp1_portawe      : in    std_logic                                := '0';
      dp1_portaaddr    : in    std_logic_vector(dp1_widthad-1 downto 0) := (others => '0');
      dp1_portadatain  : in    std_logic_vector(dp1_width-1 downto 0)   := (others => '0');
      dp1_portadataout : out   std_logic_vector(dp1_width-1 downto 0);
      dp1_portbclk     : in    std_logic                                := '0';
      dp1_portbena     : in    std_logic                                := '0';
      dp1_portbwe      : in    std_logic                                := '0';
      dp1_portbaddr    : in    std_logic_vector(dp1_widthad-1 downto 0) := (others => '0');
      dp1_portbdatain  : in    std_logic_vector(dp1_width-1 downto 0)   := (others => '0');
      dp1_portbdataout : out   std_logic_vector(dp1_width-1 downto 0);
      dp2_portaena     : in    std_logic                                := '0';
      dp2_portawe      : in    std_logic                                := '0';
      dp2_portaaddr    : in    std_logic_vector(dp0_widthad-1 downto 0) := (others => '0');
      dp2_portadatain  : in    std_logic_vector(dp0_width-1 downto 0)   := (others => '0');
      dp2_portadataout : out   std_logic_vector(dp0_width-1 downto 0);
      dp3_portaena     : in    std_logic                                := '0';
      dp3_portawe      : in    std_logic                                := '0';
      dp3_portaaddr    : in    std_logic_vector(dp1_widthad-1 downto 0) := (others => '0');
      dp3_portadatain  : in    std_logic_vector(dp1_width-1 downto 0)   := (others => '0');
      dp3_portadataout : out   std_logic_vector(dp1_width-1 downto 0));
end component;

component altddio_in
        generic (width : positive := 1;
                 power_up_high : string := "OFF");
        port (datain : in std_logic_vector(width-1 downto 0);
              inclock : in std_logic;
              inclocken : in std_logic := '1';
              aset : in std_logic := '0';
              aclr : in std_logic := '0';
              dataout_H : out std_logic_vector(width-1 downto 0);
              dataout_L : out std_logic_vector(width-1 downto 0));
end component;

component altddio_out
        generic (width : positive := 1;
                 power_up_high : string := "OFF");
        port (datain_H : in std_logic_vector(width-1 downto 0);
              datain_L : in std_logic_vector(width-1 downto 0);
              outclock : in std_logic;
              outclocken : in std_logic := '1';
              aset : in std_logic := '0';
              aclr : in std_logic := '0';
              oe : in std_logic;
              dataout : out std_logic_vector(width-1 downto 0));
end component;

component altddio_bidir
        generic(width : positive := 1;
                power_up_high : string := "OFF");
        port (datain_H : in std_logic_vector(width-1 downto 0);
              datain_L : in std_logic_vector(width-1 downto 0);
              inclock : in std_logic;
              inclocken : in std_logic := '1';
              outclock : in std_logic;
              outclocken : in std_logic := '1';
              aset : in std_logic := '0';
              aclr : in std_logic := '0';
              oe : in std_logic;
              dataout_H : out std_logic_vector(width-1 downto 0);
              dataout_L : out std_logic_vector(width-1 downto 0);
              padout : inout std_logic_vector(width-1 downto 0));
end component;

component altcdr_rx
        generic (
                number_of_channels: positive := 1;
                deserialization_factor : positive := 1;
                inclock_period : positive;
                run_length : integer := 43;
                inclock_boost : positive := 1;
                intended_device_family : string := "MERCURY");
        port (
                rx_in : in std_logic_vector(number_of_channels-1 downto 0);
                rx_inclock : in std_logic;
                rx_coreclock : in std_logic;
                rx_fifo_rden : in std_logic_vector(number_of_channels downto 0);
                rx_aclr : in std_logic;
                rx_pll_aclr : in std_logic;
                rx_out : out std_logic_vector(deserialization_factor*number_of_channels-1 downto 0);
                rx_outclock : out std_logic;
                rx_locklost : out std_logic_vector(number_of_channels-1 downto 0);
                rx_pll_locked : out std_logic;
                rx_empty : out std_logic_vector(number_of_channels-1 downto 0);
                rx_full : out std_logic_vector(number_of_channels-1 downto 0);
                rx_rlv : out std_logic_vector(number_of_channels-1 downto 0));
end component;

component altcdr_tx
        generic (
                number_of_channels: positive := 1;
                deserialization_factor : positive := 1;
                inclock_period : positive;
                inclock_boost : positive := 1;
                intended_device_family : string := "MERCURY");
        port (
                tx_in : in std_logic_vector(deserialization_factor*number_of_channels-1 downto 0);
                tx_inclock : in std_logic;
                tx_coreclock : in std_logic;
                tx_fifo_wren : in std_logic_vector(number_of_channels downto 0);
                tx_aclr : in std_logic;
                tx_pll_aclr : in std_logic;
                tx_out : out std_logic_vector(number_of_channels-1 downto 0);
                tx_outclock : out std_logic;
                tx_pll_locked : out std_logic;
                tx_empty : out std_logic_vector(number_of_channels-1 downto 0);
                tx_full : out std_logic_vector(number_of_channels-1 downto 0));
end component;

component CARRY
        port (A : in STD_LOGIC;
              Q : out STD_LOGIC);
end component;

component CASCADE
        port (A : in STD_LOGIC;
              Q : out STD_LOGIC);
end component;

component LCELL
        port (A : in STD_LOGIC;
              Q : out STD_LOGIC);
end component;

component GLOBAL
        port (A : in STD_LOGIC;
              Q : out STD_LOGIC);
end component;
end altera_mf_components;
