LIBRARY ieee;
    USE ieee.std_logic_1164.ALL;

ENTITY mem_ctrl IS
port (reset, R_W: in std_logic;
		 din:  in std_logic_vector(17 DOWNTO 0);
		 dout: out std_logic_vector(17 DOWNTO 0);
		 addr:  in std_logic_vector(16 DOWNTO 0);
		 DR: out std_logic);
END mem_ctrl;

ARCHITECTURE test OF mem_ctrl IS

    CONSTANT addr_bits : INTEGER := 17;
    CONSTANT data_bits : INTEGER := 18;
    CONSTANT tCK       : TIME    := 10 ns;
    CONSTANT HiZ       : STD_LOGIC_VECTOR (data_bits - 1 DOWNTO 0) := (OTHERS => 'Z');

    COMPONENT mt55l128l18f
        PORT (Dq       : INOUT STD_LOGIC_VECTOR (data_bits - 1 DOWNTO 0) := (OTHERS => 'Z');
              Addr     : IN    STD_LOGIC_VECTOR (addr_bits - 1 DOWNTO 0) := (OTHERS => '0');
              Lbo_n    : IN    STD_LOGIC;
              Clk      : IN    STD_LOGIC;
              Cke_n    : IN    STD_LOGIC;
              Ld_n     : IN    STD_LOGIC;
              Bwa_n    : IN    STD_LOGIC;
              Bwb_n    : IN    STD_LOGIC;
              Rw_n     : IN    STD_LOGIC;
              Oe_n     : IN    STD_LOGIC;
              Ce_n     : IN    STD_LOGIC;
              Ce2_n    : IN    STD_LOGIC;
              Ce2      : IN    STD_LOGIC;
              Zz       : IN    STD_LOGIC
        );
    END COMPONENT;
    
    FOR ALL: mt55l128l18f USE ENTITY work.mt55l128l18f(behave);
    
    SIGNAL pDq         : STD_LOGIC_VECTOR (data_bits - 1 DOWNTO 0) := (OTHERS => 'Z');
    SIGNAL pAddr       : STD_LOGIC_VECTOR (addr_bits - 1 DOWNTO 0) := (OTHERS => '0');
    SIGNAL pLbo_n      : STD_LOGIC;
    SIGNAL pClk        : STD_LOGIC;
    SIGNAL pCke_n      : STD_LOGIC;
    SIGNAL pLd_n       : STD_LOGIC;
    SIGNAL pBwa_n      : STD_LOGIC;
    SIGNAL pBwb_n      : STD_LOGIC;
    SIGNAL pRw_n       : STD_LOGIC;
    SIGNAL pOe_n       : STD_LOGIC;
    SIGNAL pCe_n       : STD_LOGIC;
    SIGNAL pCe2_n      : STD_LOGIC;
    SIGNAL pCe2        : STD_LOGIC;
    SIGNAL pZz         : STD_LOGIC;
    SIGNAL stim_done   : boolean := false;
    SIGNAL clk_done    : boolean := false;
	 signal state, nstate:integer; 
	
BEGIN
    U1 : mt55l128l18f
        PORT MAP (Dq => pDq, Addr => pAddr, Lbo_n => pLbo_n, Clk => pClk, Cke_n => pCke_n,
                  Ld_n => pLd_n, Bwa_n => pBwa_n, Bwb_n => pBwb_n,
                  Rw_n => pRw_n, Oe_n => pOe_n, Ce_n => pCe_n, Ce2_n => pCe2_n, Ce2 => pCe2, Zz => pZz);

    stimulator : PROCESS(reset,state, din, R_W)
        PROCEDURE deselect (Ld_in : STD_LOGIC;
                            Dq_in : STD_LOGIC_VECTOR (data_bits - 1 DOWNTO 0)) IS
        BEGIN
            pCe_n  <= '1' after tCK;
            pCe2_n <= '1' after tCK;
            pCe2   <= '0' after tCK;
            pZz    <= '0' after tCK;
            pLd_n  <= Ld_in after tCK;
            pRw_n  <= '0' after tCK;
            pBwa_n <= '0' after tCK;
            pBwb_n <= '0' after tCK;
            pOe_n  <= '0' after tCK;
            pCke_n <= '0' after tCK;
            pLbo_n <= '0' after tCK;
            pAddr  <= (OTHERS => '0') after tCK;
            pDq    <= Dq_in after tCK;
            
        END;

        PROCEDURE read (Lbo_in  : STD_LOGIC;
                        Ld_in   : STD_LOGIC;
                        Oe_in   : STD_LOGIC;
                        Addr_in : STD_LOGIC_VECTOR (addr_bits - 1 DOWNTO 0);
                        Dq_in   : STD_LOGIC_VECTOR (data_bits - 1 DOWNTO 0)) IS
        BEGIN
            pCe_n  <= '0' after tCK;
            pCe2_n <= '0' after tCK;
            pCe2   <= '1' after tCK;
            pZz    <= '0' after tCK;
            pLd_n  <= Ld_in after tCK;
            pRw_n  <= '1' after tCK;
            pBwa_n <= '0' after tCK;
            pBwb_n <= '0' after tCK;
            pOe_n  <= Oe_in after tCK;
            pCke_n <= '0' after tCK;
            pLbo_n <= Lbo_in after tCK;
            pAddr  <= Addr_in after tCK;
            pDq    <= Dq_in after tCK;
        END;

        PROCEDURE write (Lbo_in  : STD_LOGIC;
                         Ld_in   : STD_LOGIC;
                         Bwa_in  : STD_LOGIC;
                         Bwb_in  : STD_LOGIC;
                         Addr_in : STD_LOGIC_VECTOR (addr_bits - 1 DOWNTO 0);
                         Dq_in   : STD_LOGIC_VECTOR (data_bits - 1 DOWNTO 0)) IS
        BEGIN
            pCe_n  <= '0' after tCK;
            pCe2_n <= '0' after tCK;
            pCe2   <= '1' after tCK;
            pZz    <= '0' after tCK;
            pLd_n  <= Ld_in after tCK;
            pRw_n  <= '0' after tCK;
            pBwa_n <= Bwa_in after tCK;
            pBwb_n <= Bwb_in after tCK;
            pOe_n  <= '0' after tCK;
            pCke_n <= '0' after tCK;
            pLbo_n <= Lbo_in after tCK;
            pAddr  <= Addr_in after tCK;
            pDq    <= Dq_in after tCK;
        
        END;
    BEGIN
		DR <= '0';
     if reset='1' then 
			deselect ('0',HiZ);                                 
			nstate <= 0;
		else
			case state is
				when 0 => 
							deselect ('0',HiZ); 
							if R_W='1' then
								nstate <= 1;	
							elsif R_W='0' then
								nstate <= 3;	
							end if;
				when 1 => 
							read ('0','0','0',addr,  HiZ);
							dout<=pDq;
							nstate <= 2;
				when 2 => 
						deselect ('0',HiZ);
						DR <= '1';
						if R_W='1' then
								nstate <= 1;	
						elsif R_W='0' then
								nstate <= 3;	
						elsif reset='1' then
								nstate <= 0;	
						end if;
				when 3 => 
						write    ('0','0','0','0', addr,  HiZ);
						nstate <= 4;
				when 4 => 
						deselect ('0', din); -- data for previous write
						nstate <= state+1;
						if R_W='1' then
								nstate <= 1;	
						elsif R_W='0' then
								nstate <= 3;	
						elsif reset='1' then
								nstate <= 0;	
						end if;
				when others => null;
			end case;
		end if;
     
   END PROCESS;
                           

process (pclk)
	begin
		if pclk='1' then
			state <= nstate;
		end if;
	end process;

END test;
